unit DEditor;

interface

uses
  GL,

  DUtils,
  DDebug,
  DMath,
  DEvents,
  DWinApi,
  DSound,
  DGraph;

const
  EWidth = 640;
  EHeight = 480;

type
TEditor = class(TWinApp)
private
  FWinApi: TWinApi;
  FWindow: TWindow;
  FGraph: TGraph;

  FWidth: Integer;
  FHeight: Integer;
  FAtom: Integer;
  FObjCount: Integer;
  FCellCount: Integer;
  FCells: array of array of Cardinal;
  FObjects: array of array of Cardinal;
  FObjSprite: TSprite;
  FCellSprite: TSprite;

  FOrg: TVec2f;
  FObj: Cardinal;
  FCell: Cardinal;

  function GetActive: TVec2i;
  function IsValidActive: Boolean;
  procedure SetCell(Pos: TVec2f; Cell: Cardinal);
  procedure SetObj(Pos: TVec2f; Obj: Cardinal);

  procedure SaveLevel;
  procedure LoadLevel;
public
  constructor Create(WinApi: TWinApi);
  destructor Destroy; override;
  //   
  procedure BeginRender; inline;
  procedure Clear(R, G, B: Single; A: Single = 1); inline;
  procedure EndRender; inline;
  procedure SetBlendMode(Mode: TBlendMode); inline;
  function LoadTex(FileName: string): TTexture; inline;
  function CreateSprite: TSprite; inline;
  procedure SetTexture(ID: TTexture); inline;
  procedure SetColor(R, G, B, A: Single); inline;
  procedure DrawRect(Pos, Size: TVec2f; Fill: Boolean = True); inline;

  function GetMouse: TVec2i; inline;
  //
  procedure Render; override;
  procedure Update; override;
  procedure OnEvent(const Event: TEvent); override;

  property Mouse: TVec2i read GetMouse;
  property Active: TVec2i read GetActive;
end;

implementation

function TEditor.GetActive: TVec2i;
begin
  Result := Round2f((Mouse - FOrg) / FAtom);
end;

function TEditor.IsValidActive: Boolean;
begin
  Result := (Active.X >= 0) and (Active.Y >= 0) and
            (Active.X < FWidth) and (Active.Y < FHeight);
end;

procedure TEditor.SetCell(Pos: TVec2f; Cell: Cardinal);
begin
  if IsValidActive then
    FCells[Active.X][Active.Y] := Cell;
end;

procedure TEditor.SetObj(Pos: TVec2f; Obj: Cardinal);
begin
  if IsValidActive then
    FObjects[Active.X][Active.Y] := Obj;
end;

procedure TEditor.SaveLevel;
  var
    FileName: string;
    Data: TData;
    I, J: Integer;
begin
  FileName := GetSaveFile(FWindow, 'dlr');
  if FileName = '' then
    Exit;
  Data := TData.Create;
  Data.WriteVar(@FWidth, SizeOf(FWidth));
  Data.WriteVar(@FHeight, SizeOf(FHeight));
  for I := 0 to FWidth - 1 do
    for J := 0 to FHeight - 1 do
      Data.WriteVar(@FCells[I][J], SizeOf(FCells[I][J]));
  for I := 0 to FWidth - 1 do
    for J := 0 to FHeight - 1 do
      Data.WriteVar(@FObjects[I][J], SizeOf(FObjects[I][J]));
  Data.WriteToFile(FileName);
  Data.Free;
end;

procedure TEditor.LoadLevel;
  var
    FileName: string;
    Data: TData;
    I, J: Integer;
begin
  FileName := GetLoadFile(FWindow, 'dlr');
  if FileName = '' then
    Exit;
  Data := TData.Create(FileName);
  Data.ReadVar(@FWidth, SizeOf(FWidth));
  Data.ReadVar(@FHeight, SizeOf(FHeight));
  for I := 0 to FWidth - 1 do
    for J := 0 to FHeight - 1 do
      Data.ReadVar(@FCells[I][J], SizeOf(FCells[I][J]));
  for I := 0 to FWidth - 1 do
    for J := 0 to FHeight - 1 do
      Data.ReadVar(@FObjects[I][J], SizeOf(FObjects[I][J]));
  Data.Free;
end;

constructor TEditor.Create;
begin
  SDebug.Log('TEditor.Create');
  FWinApi := WinApi;
  FWindow := FWinApi.CreateWindow('PrizivEditor', EWidth, EHeight);
  FWindow.Subscribe(Self);
  FGraph := TGraph.Create(FWindow);

  FWidth := 16;
  FHeight := 12;
  FObjCount := 5;
  FCellCount := 5;
  FAtom := 32;
  FOrg := Vec2f(64, 64); 
  SetLength(FObjects, FWidth, FHeight);
  SetLength(FCells, FWidth, FHeight);
  FObjSprite := CreateSprite;
  FObjSprite.SetTexture('DATA\Objects.tga', 8, 8);
  FObjSprite.SetFramesBlock(0, FObjCount);
  FCellSprite := CreateSprite;
  FCellSprite.SetTexture('DATA\Cells.tga', 8, 8);
  FCellSprite.SetFramesBlock(0, FCellCount);
end;

destructor TEditor.Destroy;
begin
  FObjSprite.Free;
  FCellSprite.Free;
  FGraph.Free;
  FWindow.Free;
end;

procedure TEditor.BeginRender; inline;
begin
  FGraph.BeginRender2D(640, 480);
end;

procedure TEditor.Clear(R, G, B: Single; A: Single = 1);
begin
  FGraph.Clear(R, G, B, A);
end;

procedure TEditor.EndRender;
begin
  FGraph.EndRender;
end;

procedure TEditor.SetBlendMode(Mode: TBlendMode);
begin
  FGraph.SetBlendMode(Mode);
end;

function TEditor.LoadTex(FileName: string): TTexture; inline;
begin
  Result := FGraph.LoadTex(FileName);
end;

function TEditor.CreateSprite: TSprite; inline;
begin
  Result := FGraph.CreateSprite;
end;

procedure TEditor.SetTexture(ID: TTexture); inline;
begin
  FGraph.SetTexture(ID);
end;

procedure TEditor.SetColor(R, G, B, A: Single);  
begin
  FGraph.SetColor(R, G, B, A);  
end;

procedure TEditor.DrawRect(Pos, Size: TVec2f; Fill: Boolean = True);
begin
  FGraph.DrawRect(Pos, Size, Fill);
end;

function TEditor.GetMouse: TVec2i; 
begin
  Result := FWindow.Mouse;
end;

procedure TEditor.Render;
  var
    I, J: Integer;
begin
  BeginRender;
  Clear(0.0, 0.0, 0.0);
  SetBlendMode(bmAlpha);
  for I := 0 to FWidth-1 do
    for J := 0 to FHeight-1 do begin
      FCellSprite.StartStatic(FCells[I][J]);
      FCellSprite.Draw(Vec2f(I*FAtom, J*FAtom) + FOrg, Vec2f(FAtom, FAtom));
      FObjSprite.StartStatic(FObjects[I][J]);
      FObjSprite.Draw(Vec2f(I*FAtom, J*FAtom) + FOrg, Vec2f(FAtom, FAtom));
    end;
  FCellSprite.StartStatic(FCell);
  FCellSprite.Draw(Vec2f(EWidth-64, 64), Vec2f(64, 64));
  FObjSprite.StartStatic(FObj);
  FObjSprite.Draw(Vec2f(EWidth-64, 128), Vec2f(64, 64));
  {SetTexture(-1);
  SetColor(0.5, 0.5, 0.0, 1.0);
  DrawRect(Mouse, Vec2f(32, 32));}
  EndRender;
end;

procedure TEditor.Update;
begin
end;

procedure TEditor.OnEvent;
begin
  if Event.ID = eiKey then begin
    if Event.Key = Ord('1') then
      FCell := (FCell + 1) mod FCellCount
    else if Event.Key = Ord('2') then
      FObj := (FObj + 1) mod FObjCount
    else if Event.Key = Ord('9') then 
      SaveLevel
    else if Event.Key = Ord('0') then
      LoadLevel;
  end else if Event.ID = eiClick then begin
    if Event.Button = miLeft then
      SetCell(Event.Mouse, FCell);
    if Event.Button = miRight then
      SetObj(Event.Mouse, FObj);
  end;
end;

end.
